package com.example.trailsfinal;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.Bitmap.Config;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Typeface;
import android.graphics.Paint.Align;
import android.graphics.Path;
import android.text.format.DateFormat;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.View;
import android.graphics.Bitmap;
import android.hardware.Sensor;
import android.hardware.SensorManager;
import android.view.MotionEvent;
import android.widget.Toast;

import java.security.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import android.os.Handler;
import android.os.SystemClock;
import android.os.Environment;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import android.util.Pair;


public class DrawingView extends View{
	
	
	//drawing path
	private Path drawPath;
	//drawing and canvas paint
	private Paint drawPaint, canvasPaint, delPaint, circlePaint;
	//initial color
	private int paintColor = Color.BLACK;
	//canvas
	private Canvas drawCanvas;

	//canvas bitmap
	private Bitmap canvasBitmap;
	
	private int liftOccured = 0;
	
	//The paths the user drew inside the circle
	private List<Path> lines = new ArrayList();
	
	//save the old bitmap 
	private Bitmap oldCanvasBitmap;
	
	//The paint for the text
	private Paint textPaint;
	
	private Paint errPaint;
	
	//any time in seconds longer than this will be considered a pause
	private double pauseThreshold = 0.1;
	
	//the graph type
	private String type;
	
	private ArrayList<String> messages;
	
	//indicates that the node has been entered 
	//switches off 
	private int enterFlag = 0;
	private float rad = 35;
	//for time, switches off 
	private int startFlag = 0;
	private Node previousEnterNode = null;
	
	private DisplayMetrics dm;
		
	private float sx, sy, ex, ey;
	
	private float touchX = -1, touchY = -1;
	private float oldX = -1, oldY = -1, oldLiftX = -1, oldLiftY = -1;
	private float touchFlag = -1;
	private long lastTimeTouched = -1;
	
	private float lux;
	
	private long startLift = -1;
	
	private ArrayList<Path> drawnPaths = new ArrayList<Path>();		
	Node test;
	
	Context cont;

	
	private Graph myGraph;
	private int pointerID;
	private Bitmap unmarkedBitmap;
	 
	//TODO: start time will be as soon as the Start btn is clicked??
	public DrawingView(Context context) {
	 super(context);
	 setupDrawing();
	 cont = context;
	}
	 
	public DrawingView(Context context, AttributeSet attrs) {
	 super(context, attrs);
	 setupDrawing();
	 cont = context;
	}
	 
	public DrawingView(Context context, AttributeSet attrs, int defStyle) {
	 super(context, attrs, defStyle);
	 setupDrawing();
	 cont = context;
	}
	
	public void setType(String atype)
	{
		type = atype;
	}
	
	//setup drawing
	private void setupDrawing(){

		//prepare for drawing and setup paint stroke properties
		drawPath = new Path();
		drawPaint = new Paint();
		drawPaint.setColor(Color.GRAY);
		drawPaint.setAntiAlias(true);
		drawPaint.setStrokeWidth(5);
		drawPaint.setStyle(Paint.Style.STROKE);
		drawPaint.setStrokeJoin(Paint.Join.ROUND);
		drawPaint.setStrokeCap(Paint.Cap.ROUND);
		
		circlePaint = new Paint();
		circlePaint.setColor(paintColor);
		circlePaint.setAntiAlias(true);
		circlePaint.setStrokeWidth(3);
		circlePaint.setStyle(Paint.Style.STROKE);
		circlePaint.setStrokeJoin(Paint.Join.ROUND);
		circlePaint.setStrokeCap(Paint.Cap.ROUND);
		
		//The font for the numerals and text inside the nodes
		Typeface tf = Typeface.create("Times New Roman", Typeface.NORMAL);
		
		textPaint = new Paint();
		textPaint.setTypeface(tf);
		textPaint.setColor(paintColor);
		textPaint.setStrokeWidth(7);
		textPaint.setTextSize(45);
		textPaint.setTextAlign(Align.CENTER);
		
		errPaint = new Paint();
		errPaint.setColor(Color.RED);
		errPaint.setStrokeWidth(5);
		
		delPaint = new Paint();
		delPaint.setColor(Color.WHITE);
		delPaint.setStrokeWidth(5);
		
		canvasPaint = new Paint(Paint.DITHER_FLAG);
		
		
		
	}
	
	public void setMessages(ArrayList<String> msgs)
	{
		messages = msgs;
	}
	
	public void makeGraph()
	{
		myGraph = new Graph(type, drawCanvas.getWidth(), drawCanvas.getHeight(), dm, cont);
		Node node;
		List<Node> nodes = myGraph.getNodes();
		
		//System.out.println(nodes.size());
		for(int i = 0; i < nodes.size(); i++)
		{
			node = nodes.get(i);
			//System.out.println(node.getCharacter());
			//Log.i("Trails", node.getCharacter());
			//System.out.println(node.getX());
			//System.out.println(node.getY());
			node.draw(drawCanvas, circlePaint, textPaint);
		}
		
		sx = nodes.get(0).getX();
		sy = nodes.get(0).getY() - 50;
		
		ex = nodes.get(nodes.size() - 1).getX();
		ey = nodes.get(nodes.size() - 1).getY() - 50;
		
		drawCanvas.drawText("Begin", sx, sy, textPaint);
		drawCanvas.drawText("End", ex, ey, textPaint);
		
		
	}
	
	@Override
	protected void onSizeChanged(int w, int h, int oldw, int oldh) {
		super.onSizeChanged(w, h, oldw, oldh);
		canvasBitmap = Bitmap.createBitmap(w, h, Bitmap.Config.ARGB_8888);
		drawCanvas = new Canvas(canvasBitmap);
		drawCanvas.drawColor(Color.WHITE);
		if(type != null)
		{
			makeGraph();
		}
		
		unmarkedBitmap = Bitmap.createBitmap(w, h, Bitmap.Config.ARGB_8888);
		unmarkedBitmap = canvasBitmap.copy(canvasBitmap.getConfig(), canvasBitmap.isMutable());
		
	}
	
	public Node checkCircles(float x, float y)
	{
		Node n;
		List <Node> nodes;
		nodes = myGraph.getNodes();
		for(int i = 0; i < nodes.size(); i++)
		{
			n = nodes.get(i);
			if(n.isWithin(x, y))
			{
				return n;
			}
			
		}
		n = new Node(0,0,0,"",-1,"");
		
		return n;
	}
	
	public void setMetrics(DisplayMetrics m)
	{
		dm = m;
	}
	
	//Maybe only set the node regions, then some kind of onTouch
	 
	@Override
	protected void onDraw(Canvas canvas) {

	canvas.drawBitmap(canvasBitmap, 0, 0, canvasPaint);
	canvas.drawPath(drawPath, drawPaint);

	}
	

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		
		RateInfo rdi;
		
		pointerID = event.getPointerId(event.getActionIndex());
		
		if(event.getActionIndex() == 0 && pointerID == 0)
		{
		
		//Log.i("POINTER: ", String.valueOf(event.getActionIndex()));
		long timeTouched = System.nanoTime();
		
		float pressure = -1000000;
		float size = -1000000;
		
		if(startFlag == 1)
		{
		
			SensorManager sManager = (SensorManager) cont.getSystemService(Context.SENSOR_SERVICE);
			
			Sensor light = sManager.getDefaultSensor(Sensor.TYPE_LIGHT);
			//Log.i("LIGHT: ", String.valueOf(light.getMaximumRange()));
	
			myGraph.setNumTouchPoints(myGraph.getNumTouchPoints() + 1);
			pressure = event.getPressure();
			
			myGraph.setPressureSum(myGraph.getPressureSum() + pressure);
			myGraph.addPressureSample(pressure);
			
			size = event.getSize();
			myGraph.setSizeSum(myGraph.getSizeSum() + size);
			myGraph.addSizeSample(size);
		}
	
		
		touchX = event.getX();
		touchY = event.getY();
		
		//if inside a node
		if(enterFlag == 1)
		{
			if(startFlag == 1)
			{
				if(myGraph.getPressureDataInside().size() != 0)
				{
					AvgTypeData p = myGraph.getPressureDataInside().get(myGraph.getPressureDataInside().size() - 1);
					p.setNumSamples(p.getNumSamples() + 1);
					p.setSumSamples(p.getSumSamples() + pressure);
					p.addSample(pressure);
					
					AvgTypeData s = myGraph.getSizeDataInside().get(myGraph.getSizeDataInside().size() - 1);
					s.setNumSamples(s.getNumSamples() + 1);
					s.setSumSamples(s.getSumSamples() + size);
					s.addSample(size);
				}
			}
		}
		else
		{
			
			if(startFlag == 1)
			{
				if(myGraph.getPressureDataBetween().size() == 0)
				{
					
					if(previousEnterNode == null)
					{
						myGraph.addPressureDataBetween("", "BN");
						myGraph.addSizeDataBetween("", "BN");
						myGraph.addRateDataBetween("", touchX, touchY, timeTouched, "BN");
					}
					else
					{
						myGraph.addPressureDataBetween(previousEnterNode.getCharacter(), "BN");
						myGraph.addSizeDataBetween(previousEnterNode.getCharacter(), "BN");
						myGraph.addRateDataBetween(previousEnterNode.getCharacter(), touchX, touchY, timeTouched, "BN");
					}
					
				}
				
				AvgTypeData p = myGraph.getPressureDataBetween().get( myGraph.getPressureDataBetween().size() - 1);
				p.setNumSamples(p.getNumSamples() + 1);
				p.setSumSamples(p.getSumSamples() + pressure);
				p.addSample(pressure);
				
				AvgTypeData s = myGraph.getSizeDataBetween().get(myGraph.getSizeDataBetween().size() - 1);
				s.setNumSamples(s.getNumSamples() + 1);
				s.setSumSamples(s.getSumSamples() + size);
				p.addSample(size);
				}
			
		}
		
		
		
		//respond to down, move and up events
		switch (event.getAction()) {
		case MotionEvent.ACTION_DOWN:
			

			oldX = touchX;
			oldY = touchY;

			
			if(startLift != -1)
			{
				if(startFlag == 1)
				{
					myGraph.addLiftTime(startLift, timeTouched, oldLiftX, oldLiftY);
				}

				startLift = -1;
			}
			
			lastTimeTouched = timeTouched;
			touchFlag = 1;
			drawPath.moveTo(touchX, touchY);
			liftOccured = 0;
			
			break;
		case MotionEvent.ACTION_MOVE:
			drawPath.lineTo(touchX, touchY);
			drawCanvas.drawPath(drawPath, drawPaint);
			
			if(startFlag == 1)
			{
				if(oldX != -1 && oldY != -1)
				{
	
					if(touchX > oldX + 10 || touchX < oldX - 10 || touchY > oldY + 10 || touchY < oldY - 10)
					{
						if(lastTimeTouched != -1)
						{
							if(((timeTouched - lastTimeTouched)/1000000000.0) > pauseThreshold)
							{
								Log.i("StartTime", String.valueOf(myGraph.getStartTime()));
								Log.i("Time", "Pause");
								Log.i("TimeTouched", String.valueOf(timeTouched /1000000000.0));
								Log.i("LTimeTouched", String.valueOf(lastTimeTouched/1000000000.0));
								
								myGraph.addPauseTime(lastTimeTouched, timeTouched, touchX, touchY);
								PauseTime p = myGraph.getPauseTimes().get(myGraph.getPauseTimes().size() - 1);
								p.setPressure(pressure);
								p.setSize(size);
							
							}
						}
						lastTimeTouched = timeTouched;
						oldX = touchX;
						oldY = touchY;
					}
					
				}
			}

			break;
		case MotionEvent.ACTION_UP:
			
			if(startFlag == 1)
			{
				liftOccured = 1;
				drawPath.lineTo(touchX, touchY);
				if(lastTimeTouched != -1)
				{
					if(((timeTouched - lastTimeTouched)/1000000000.0) > pauseThreshold)
					{
						Log.i("Time", "Pause");
						Log.i("PAUSE", String.valueOf(lastTimeTouched / 1000000000.0));
						myGraph.addPauseTime(lastTimeTouched, timeTouched, touchX, touchY);
						
						
						PauseTime p = myGraph.getPauseTimes().get(myGraph.getPauseTimes().size() - 1);
						p.setPressure(pressure);
						p.setSize(size);
					
					}
					lastTimeTouched = timeTouched;
				}
				
				startLift = timeTouched;
				oldLiftX = touchX;
			oldLiftY = touchY;
			}
			
			drawCanvas.drawPath(drawPath, drawPaint);
			drawPath.reset();
			
			Log.i("Trails", "Liftup");
			break;
		default:
			return false;
		}
		
		
		
		Node n = checkCircles(touchX,touchY);
		
		//if there's a new node reset
		//this is for error checking make sure a bunch of errors don't occur until new node is entered
		if(previousEnterNode != null)
		{
			if(!n.getCharacter().equals(previousEnterNode.getCharacter()))
			{
				enterFlag = 0;
				//once they leave the circle clear all the lines
				lines.clear();
				//TODO: redraw the lines inside the circle here
				
				
				if(previousEnterNode.getTimeInfo().size() > 0)
				{
					if(previousEnterNode.getTimeInfo().get(previousEnterNode.getTimeInfo().size() -1).getExitTimestamp() == -1)
					{
						if(startFlag == 1)
						{

							previousEnterNode.getTimeInfo().get(previousEnterNode.getTimeInfo().size() -1).setExitTimestamp(timeTouched);
							myGraph.addPressureDataBetween(previousEnterNode.getCharacter(), "BN");
							myGraph.addSizeDataBetween(previousEnterNode.getCharacter(), "BN");
							myGraph.addRateDataBetween(previousEnterNode.getCharacter(), touchX, touchY, timeTouched, "BN");
						}
					}
					
					if(previousEnterNode.getTimeInfo().get(previousEnterNode.getTimeInfo().size() -1).getExitX() == -1)
					{
						
						previousEnterNode.getTimeInfo().get(previousEnterNode.getTimeInfo().size() -1).setExitLocation(touchX, touchY);
					}
				}
				n.draw(drawCanvas, circlePaint, textPaint);
			}
		}
		
		
		
	
		if(n.getIndex() != -1 && myGraph.trailsComplete(myGraph.getNumCorrect()) == 0.0 && enterFlag != 1)
		{
			
		
			n.setHit(n.getHit() + 1);
			
			//record enter time here
			previousEnterNode = n;
			//TODO: don't need previous node
			n.addTimeEntered(timeTouched, previousEnterNode.getCharacter(), touchX, touchY);
			
			enterFlag = 1;
			
			if(startFlag == 1)
			{
				AvgTypeData p = myGraph.getPressureDataBetween().get( myGraph.getPressureDataBetween().size() - 1);
				p.setEndNode(n.getCharacter());
				
				AvgTypeData s = myGraph.getSizeDataBetween().get(myGraph.getSizeDataBetween().size() - 1);
				s.setEndNode(n.getCharacter());
				
				RateInfo r = myGraph.getRateDataBetween().get(myGraph.getRateDataBetween().size() - 1);
				r.setEndTime(timeTouched);
				r.setEndNode(n.getCharacter());
				r.setEndLocation(touchX, touchY);
				
				myGraph.addPressureDataInside(n.getCharacter(), "N");
				myGraph.addSizeDataInside(n.getCharacter(), "N");
			}

			//check if everything is done
			if(myGraph.inOrder(n, myGraph.getNumCorrect()))
			{
				myGraph.setNumCorrect(n, myGraph.getNumCorrect());
				
				long endTime = myGraph.trailsComplete(myGraph.getNumCorrect());
				
				if(endTime != 0)
				{
					myGraph.getDuration(endTime);

						//notify user
						AlertDialog.Builder builder = new AlertDialog.Builder(this.getContext());
						builder.setMessage("Complete!");
						builder.setPositiveButton("Continue", new DialogInterface.OnClickListener() {
							
							
							@Override
							public void onClick(DialogInterface dialog, int which) {
								
								writeResultsToFile();
								savePlainImage();
								saveImage();
								if(!myGraph.getType().equals("PA") && !myGraph.getType().equals("PB") && !myGraph.getType().equals("PS") && !myGraph.getType().equals("PS2"))
								{
									((Activity) cont).finish();
								}
								
							}
						});
						
						AlertDialog dialog = builder.create();
						dialog.show();
				
					
				}
				
				//Here only set startFlag if it's the 1 circle
				if(startFlag != 1 && n.getIndex() == 0)
				{
					myGraph.setStartTime(timeTouched);
					startFlag = 1;
				}
				
				
				
			}
			else
			{
				
				runError(n);
	
			}
		}
		
		//redraw
		invalidate();
		}
		return true;
		
	}
	
	private void runError(Node n)
	{
		
		//TODO: first draw the line 
		//save the old bitmap 
		//oldCanvasBitmap = canvasBitmap.copy(canvasBitmap.getConfig(), canvasBitmap.isMutable());
		
		
		//TODO: sets pixels (very tiny)
		//canvasBitmap.setPixel(100, 100, Color.RED);
		//canvasBitmap.setPixel(101, 100, Color.RED);
		//canvasBitmap.setPixel(102, 100, Color.RED);
		//drawCanvas = new Canvas(canvasBitmap);
		//this.invalidate();
		//drawCanvas.drawBitmap(canvasBitmap, 0, 0, canvasPaint);
		
		//TODO: gets number of colored pixels
		/*
		int count = 0;
		for(int i = 0; i < pixels.length; i++)
		{
			if(pixels[i] == Color.GRAY)
			{
				count++;
			}
		}
		Log.i("GrayPIXELS", String.valueOf(count));*/
		
		
		//TODO: save gray pixels already there
		
		
		int[] pixels = new int[canvasBitmap.getWidth() * canvasBitmap.getHeight()];
		
		canvasBitmap.getPixels(pixels, 0, canvasBitmap.getWidth(), 0, 0, canvasBitmap.getWidth(), canvasBitmap.getHeight());
		Log.i("PIXELS", String.valueOf(pixels.length));
		
		int diameter = (int)(n.getRadius()*2);
		int x = (int) (n.getX() - n.getRadius());
		int y = (int) (n.getY() - n.getRadius());

		int[] circlePixels = new int[(int) (diameter*diameter)];
		canvasBitmap.getPixels(circlePixels, 0, diameter, x, y, (int)diameter, (int)diameter);
		
		//do the X thing
  	    float startx1 = 0, starty1 = 0, endx1 = 0, endy1 = 0;
  	    float startx2 = 0, starty2 = 0, endx2 = 0, endy2 = 0;
		
		//calculate position
		startx1 = (float) (n.getX() + Math.cos(Math.toRadians(45.0)) * rad);
		starty1 = (float) (n.getY() - Math.sin(Math.toRadians(45.0)) * rad);
		
		endx1 = (float) (n.getX() + Math.cos(Math.toRadians(225.0)) * rad);
		endy1 = (float) (n.getY() - Math.sin(Math.toRadians(225.0)) * rad); 
		
		//draw the first error line
		drawCanvas.drawLine(startx1, starty1, endx1, endy1, errPaint);
		
		//calculate position
		startx2 = (float) (n.getX() + Math.cos(Math.toRadians(135.0)) * rad);
		starty2 = (float) (n.getY() - Math.sin(Math.toRadians(135.0)) * rad);
		
		endx2 = (float) (n.getX() + Math.cos(Math.toRadians(315.0)) * rad);
		endy2 = (float) (n.getY() - Math.sin(Math.toRadians(315.0)) * rad); 
		
		//draw the second error line
		drawCanvas.drawLine(startx2, starty2, endx2, endy2, errPaint);

		myGraph.setNumErrors(myGraph.getNumErrors() + 1);
		//show an X for about 1 second
		eraseHandler.postDelayed(new EraseRunnable(startx1, starty1, endx1, endy1, startx2, starty2, endx2, endy2, n, delPaint, drawCanvas, this, textPaint, drawPaint, circlePaint, drawPath, oldCanvasBitmap, canvasBitmap, circlePixels), 1000);
		
	}
	
	private Handler eraseHandler = new Handler();
	
	public void saveImage()
	{
		//File dir = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS), "Results");
		int r = 3;
		Bitmap oldBit = canvasBitmap.copy(canvasBitmap.getConfig(), canvasBitmap.isMutable());
		
		
		//TODO: lift location is wrong, is time wrong???
		for(int i = 0; i < myGraph.getLiftTimes().size(); i++)
		{
			PauseTime time = myGraph.getLiftTimes().get(i);
			float x = time.getX();
			float y = time.getY();
			Paint p = new Paint();
			p.setColor(Color.BLUE);
			drawCanvas.drawCircle(x, y, 5, p);
			this.invalidate();
		}
		
		for(int i = 0; i < myGraph.getPauseTimes().size(); i++)
		{
			PauseTime time = myGraph.getPauseTimes().get(i);
			float x  = time.getX();
			float y = time.getY();
			
			Paint p = new Paint();
			p.setColor(Color.RED);
			drawCanvas.drawCircle(x, y, 3, p);
			this.invalidate();
		}
		
		String file;
		if( !"".equals(messages.get(0)))
		{
			file = "/storage/extSdCard/Results/" + messages.get(0); 
		}
		else
		{
			file = "/storage/extSdCard/Results/Test";
		}
		
		File dirid = new File(file);
		if(!dirid.exists())
		{
			//then try output to internal
			File indir = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS), "Results");
			if(!indir.mkdirs())
			{
				Log.i("Trails", "Directory not created");
				//return -1;
			}
			
			dirid = indir;
		}
		
		String nm = "drawing_";
		nm += messages.get(0);
		nm += "_";
		nm += myGraph.getType();
		nm += "_";
		Calendar c = Calendar.getInstance();
		String m = String.valueOf(c.get(Calendar.MONTH));
		nm += m;
		nm += "_";
		String d = String.valueOf(c.get(Calendar.DATE));
		nm += d;
		nm += "_";
		String y = String.valueOf(c.get(Calendar.YEAR));
		nm += y;
		nm += "_";
		String h = String.valueOf(c.get(Calendar.HOUR));
		nm += h;
		nm += "_";
		String min = String.valueOf(c.get(Calendar.MINUTE));
		nm += min;
		nm += "_";
		String sec = String.valueOf(c.get(Calendar.SECOND));
		nm += sec;
		nm += ".png";
		
		File test = new File(dirid, nm);
		//File newtest = new File(dirid, "unmarked"+myGraph.getType()+".png");
		try {
			FileOutputStream f = new FileOutputStream(test);
			canvasBitmap.compress(CompressFormat.PNG, 80, f);
			f.close();
			canvasBitmap = oldBit;
			this.invalidate();
			
			
			//FileOutputStream g = new FileOutputStream(newtest);
			//unmarkedBitmap.compress(CompressFormat.PNG, 80, g);
			//g.close();
			
			

		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		
		
		
	}
	
	public void savePlainImage()
	{
	
		String file;
		if( !"".equals(messages.get(0)))
		{
			file = "/storage/extSdCard/Results/" + messages.get(0); 
		}
		else
		{
			file = "/storage/extSdCard/Results/Test";
		}
		
		File dirid = new File(file);
		if(!dirid.mkdirs())
		{
			Log.i("Trails", "ID Directory not created");
			//return -1;
		}
		
		if(!dirid.exists())
		{
			//then try output to internal
			File indir = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS), "Results");
			if(!indir.mkdirs())
			{
				Log.i("Trails", "Directory not created");
				//return -1;
			}
			
			dirid = indir;
		}
		
		String nm = "drawingplain_";
		nm += messages.get(0);
		nm += "_";
		nm += myGraph.getType();
		nm += "_";
		Calendar c = Calendar.getInstance();
		String m = String.valueOf(c.get(Calendar.MONTH));
		nm += m;
		nm += "_";
		String d = String.valueOf(c.get(Calendar.DATE));
		nm += d;
		nm += "_";
		String y = String.valueOf(c.get(Calendar.YEAR));
		nm += y;
		nm += "_";
		String h = String.valueOf(c.get(Calendar.HOUR));
		nm += h;
		nm += "_";
		String min = String.valueOf(c.get(Calendar.MINUTE));
		nm += min;
		nm += "_";
		String sec = String.valueOf(c.get(Calendar.SECOND));
		nm += sec;
		nm += ".png";
		
		File test = new File(dirid, nm);
		try {
			FileOutputStream f = new FileOutputStream(test);
			canvasBitmap.compress(CompressFormat.PNG, 80, f);
			f.close();
			this.invalidate();

		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		
		
		
	}
	
	public void writeResultsToFile()
	{
		
		String state = Environment.getExternalStorageState();
		Log.i("Trails", "infile");
		Log.i("Trails", state);
		
		if(Environment.MEDIA_MOUNTED_READ_ONLY.equals(state)){
			Log.i("Trails", "ONLY READ");
		
		}
		if(Environment.MEDIA_MOUNTED.equals(state)){
			Log.i("Trails", "SD card Mounted");
			//on external public dir
			//may not be able to say null on the subdir
			
			//File sdcarddir = Environment.getExternalStorageDirectory();
			
			//
			//Log.i("Trails", sdcarddir.getAbsolutePath());
			
			File dir = new File("/storage/extSdCard/Results");
			if(!dir.mkdirs())
			{
				Log.i("Trails", "Directory not created");
				//return -1;
			}
			
			String file;
			if( !"".equals(messages.get(0)))
			{
				file = "/storage/extSdCard/Results/" + messages.get(0); 
			}
			else
			{
				//TODO: note that this dir is specific to the galaxy device
				file = "/storage/extSdCard/Results/Test";
			}
			
			
			File dirid = new File(file);
			if(!dirid.mkdirs())
			{
				Log.i("Trails", "ID Directory not created");
				
			}
			
			if(!dirid.exists())
			{
				//then try output to internal
				File indir = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS), "Results");
				if(!indir.mkdirs())
				{
					Log.i("Trails", "Directory not created");
					//return -1;
				}

				dirid = indir;
			}
			
		
			String nm = "results_";
			nm += messages.get(0);
			nm += "_";
			nm += myGraph.getType();
			nm += "_";
			Calendar c = Calendar.getInstance();
			String m = String.valueOf(c.get(Calendar.MONTH)+1);
			nm += m;
			nm += "_";
			String d = String.valueOf(c.get(Calendar.DATE));
			nm += d;
			nm += "_";
			String y = String.valueOf(c.get(Calendar.YEAR));
			nm += y;
			nm += "_";
			String h = String.valueOf(c.get(Calendar.HOUR));
			nm += h;
			nm += "_";
			String min = String.valueOf(c.get(Calendar.MINUTE));
			nm += min;
			nm += "_";
			String sec = String.valueOf(c.get(Calendar.SECOND));
			nm += sec;
			nm += ".csv";
			File test = new File(dirid, nm);
			
			
			try {
				FileOutputStream f = new FileOutputStream(test);
				Log.i("Trails", "afterstream");
				PrintWriter pw = new PrintWriter(f);
				
				//Headers
				List<Node> nodes = myGraph.getNodes();
				
				pw.print("ID_" + myGraph.getType() + ",");
				pw.print("Birthdate_" + myGraph.getType() + ",");
				pw.print("Age_" + myGraph.getType() + ",");
				pw.print("Education_" + myGraph.getType() + ",");
				pw.print("Gender_" + myGraph.getType() + ",");
				pw.print("Handedness_" + myGraph.getType() + ",");
				
				pw.print("Total Time s_" + myGraph.getType() + ",");
				pw.print("Number Errors_" + myGraph.getType() + ",");
				
				for(int i = 0; i < nodes.size(); i++)
				{
					Node node = nodes.get(i);
					String hitStr = node.getCharacter();
					hitStr += "_";
					hitStr += "NumTimesTouched_";
					hitStr += myGraph.getType();
					
					pw.print(hitStr);
					pw.print(",");
				}
				
				//Enter and exit timestamps
				for(int i = 0; i < nodes.size(); i++)
				{
					Node node = nodes.get(i);
					//Enter time headers
					for(int j = 0; j < node.getTimeInfo().size(); j++)
					{
						String enterStr = node.getCharacter();
						enterStr += "_";
						enterStr += "EnterTime_";
						enterStr += String.valueOf(j+1);
						enterStr += "_";
						enterStr += myGraph.getType();
						
						pw.print(enterStr);
						pw.print(",");
						
						enterStr = node.getCharacter();
						enterStr += "_";
						enterStr += "EnterLocation_";
						enterStr += String.valueOf(j+1);
						enterStr += "_";
						enterStr += myGraph.getType();
						pw.print(enterStr);
						pw.print(",");
					}
					
					//Exit time headers
					for(int j = 0; j < node.getTimeInfo().size(); j++)
					{
						String exitStr = node.getCharacter();
						exitStr += "_";
						exitStr += "ExitTime_";
						exitStr += String.valueOf(j+1);
						exitStr += "_";
						exitStr += myGraph.getType();
						pw.print(exitStr);
						pw.print(",");
						
						exitStr = node.getCharacter();
						exitStr += "_";
						exitStr += "ExitLocation_";
						exitStr += String.valueOf(j+1);
						exitStr += "_";
						exitStr += myGraph.getType();
						pw.print(exitStr);
						pw.print(",");
					}
					
					
				}
				
				for(int i = 0; i < myGraph.getLiftTimes().size(); i++)
				{
					String lift = "LifttimeStart_";
					lift += String.valueOf(i+1);
					lift += "_";
					lift += myGraph.getType();
					pw.print(lift);
					pw.print(",");
					lift = "LifttimeEnd_";
					lift += String.valueOf(i+1);
					lift += "_";
					lift += myGraph.getType();
					pw.print(lift);
					pw.print(",");
					lift = "LifttimeLocation_";
					lift += String.valueOf(i+1);
					lift += "_";
					lift += myGraph.getType();
					pw.print(lift);
					pw.print(",");
				}
				
				for(int i = 0; i < myGraph.getPauseTimes().size(); i++)
				{
					String lift = "PauseTimeStart_";
					lift += String.valueOf(i+1);
					lift += "_";
					lift += myGraph.getType();
					pw.print(lift);
					pw.print(",");
					lift = "PauseTimeEnd_";
					lift += String.valueOf(i+1);
					lift += "_";
					lift += myGraph.getType();
					pw.print(lift);
					pw.print(",");
					lift = "PauseTimeLocation_";
					lift += String.valueOf(i+1);
					lift += "_";
					lift += myGraph.getType();
					pw.print(lift);
					pw.print(",");
					lift = "PauseTimePressure_";
					lift += String.valueOf(i+1);
					lift += "_";
					lift += myGraph.getType();
					pw.print(lift);
					pw.print(",");
					lift = "PauseTimeSize_";
					lift += String.valueOf(i+1);
					lift += "_";
					lift += myGraph.getType();
					pw.print(lift);
					pw.print(",");
				}
				
				pw.print("TotalAvgPressure_" + myGraph.getType());
				pw.print(",");
				pw.print("TotalAvgSize_" + myGraph.getType());
				pw.print(",");
				pw.print("TotalVariancePressure_" + myGraph.getType());
				pw.print(",");
				pw.print("TotalVarianceSize_" + myGraph.getType());
				pw.print(",");
				
				
				for(int i = 0; i < myGraph.getNodes().size(); i++)
				{
					Node n = myGraph.getNodes().get(i);
					for(int j = 0; j < n.getTimeInfo().size(); j++)
					{
						String rate = "RateDataInside_";
						rate += n.getCharacter();
						rate += "_";
						rate += String.valueOf(j+1);
						rate += "_";
						rate += myGraph.getType();
						pw.print(rate);
						pw.print(",");
						rate = "DistanceInside_";
						rate += n.getCharacter();
						rate += "_";
						rate += String.valueOf(j+1);
						rate += "_";
						rate += myGraph.getType();
						pw.print(rate);
						pw.print(",");
					}
				}
				
				for(int i = 0; i < myGraph.getPressureDataInside().size(); i++)
				{
					AvgTypeData p = myGraph.getPressureDataInside().get(i);
					String press = "AvgPressureInsideNode_";
					press += p.getStartNode();
					press += "_";
					press += myGraph.getType();
					pw.print(press);
					pw.print(",");
					press = "VariancePressureInsideNode_";
					press += p.getStartNode();
					press += "_";
					press += myGraph.getType();
					pw.print(press);
					pw.print(",");
				}
				
				for(int i = 0; i < myGraph.getSizeDataInside().size(); i++)
				{
					AvgTypeData s = myGraph.getSizeDataInside().get(i);
					String size = "AvgSizeInsideNode_";
					size += s.getStartNode();
					size += "_";
					size += myGraph.getType();
					pw.print(size);
					pw.print(",");
					size = "VarianceSizeInsideNode_";
					size += s.getStartNode();
					size += "_";
					size += myGraph.getType();
					pw.print(size);
					pw.print(",");
				}
				
				for(int i =0; i < myGraph.getPressureDataBetween().size(); i++)
				{
					AvgTypeData p = myGraph.getPressureDataBetween().get(i);
					String pressure = "AvgPressureBetween_";
					pressure += p.getStartNode();
					pressure += "_";
					pressure += p.getEndNode();
					pressure += "_";
					pressure += myGraph.getType();
					pw.print(pressure);
					pw.print(",");
					pressure = "VariancePressureBetween_";
					pressure += p.getStartNode();
					pressure += "_";
					pressure += p.getEndNode();
					pressure += "_";
					pressure += myGraph.getType();
					pw.print(pressure);
					pw.print(",");
				}
				
				for(int i = 0; i < myGraph.getSizeDataBetween().size(); i++)
				{
					AvgTypeData s = myGraph.getSizeDataBetween().get(i);
					String size = "AvgSizeBetween_";
					size += s.getStartNode();
					size += "_";
					size += s.getEndNode();
					size += "_";
					size += myGraph.getType();
					pw.print(size);
					pw.print(",");
					size = "VarianceSizeBetween_";
					size += s.getStartNode();
					size += "_";
					size += s.getEndNode();
					size += "_";
					size += myGraph.getType();
					pw.print(size);
					pw.print(",");
				}
				
				for(int i = 0; i < myGraph.getRateDataBetween().size(); i++)
				{
					RateInfo r = myGraph.getRateDataBetween().get(i);
					String rate = "RateBetween_";
					rate += r.getStartNode();
					rate += "_";
					rate += r.getEndNode();
					rate += "_";
					rate += myGraph.getType();
					pw.print(rate);
					pw.print(",");
					rate = "DistanceBetween_";
					rate += r.getStartNode();
					rate += "_";
					rate += r.getEndNode();
					rate += "_";
					rate += myGraph.getType();
					pw.print(rate);
					pw.print(",");
					
				}
				
				//Data
				pw.println();
				for(int i = 0; i < messages.size(); i++)
				{
						pw.print(messages.get(i)+",");
				}
				
				pw.print(myGraph.getTotalTime());
				pw.print(",");
				pw.print(myGraph.getNumErrors());
				pw.print(",");
				
				
				
				//print the circles Circle_1_NumTimesHit
				//print Circle_1_Enter_1 all the enter times
				//print Circle_1_Exit_1 all the exit times for that circle
				
				//another file with A to B stuff
				
				
				for(int i = 0; i < nodes.size(); i++)
				{
					Node node = nodes.get(i);
					pw.print(node.getHit());
					pw.print(",");
				}
				
				//Enter and exit timestamps
				for(int i = 0; i < nodes.size(); i++)
				{
					Node node = nodes.get(i);
					//Enter time data
					for(int j = 0; j < node.getTimeInfo().size(); j++)
					{
						double time = (node.getTimeInfo().get(j).getEnterTimestamp() - myGraph.getStartTime())/1000000000.0;
						pw.print(String.valueOf(time));
						pw.print(",");
						
						//location
						float ex = node.getTimeInfo().get(j).getEnterX();
						float ey = node.getTimeInfo().get(j).getEnterY();
						
						pw.print(String.valueOf(ex));
						pw.print("|");
						pw.print(String.valueOf(ey));
						pw.print(",");
					}
					//Exit time data
					for(int j = 0; j < node.getTimeInfo().size(); j++)
					{
						double time = (node.getTimeInfo().get(j).getExitTimestamp() - myGraph.getStartTime())/1000000000.0;
						pw.print(String.valueOf(time));
						pw.print(",");
						
						//location
						float ex = node.getTimeInfo().get(j).getExitX();
						float ey = node.getTimeInfo().get(j).getExitY();
						
						pw.print(String.valueOf(ex));
						pw.print("|");
						pw.print(String.valueOf(ey));
						pw.print(",");
					}
					
					
				}
				
				for(int i = 0; i < myGraph.getLiftTimes().size(); i++)
				{
					double stime = (myGraph.getLiftTimes().get(i).getStart() - myGraph.getStartTime())/1000000000.0;
					double etime = (myGraph.getLiftTimes().get(i).getEnd() - myGraph.getStartTime())/1000000000.0;
					
					float lx = myGraph.getLiftTimes().get(i).getX();
					float ly = myGraph.getLiftTimes().get(i).getY();
					
					pw.print(String.valueOf(stime));
					pw.print(",");
					pw.print(String.valueOf(etime));
					pw.print(",");
					pw.print(String.valueOf(lx));
					pw.print("|");
					pw.print(String.valueOf(ly));
					pw.print(",");
					
				}
				
				for(int i = 0; i < myGraph.getPauseTimes().size(); i++)
				{
					double stime = (myGraph.getPauseTimes().get(i).getStart() - myGraph.getStartTime())/1000000000.0;
					double etime = (myGraph.getPauseTimes().get(i).getEnd() - myGraph.getStartTime())/1000000000.0;
					
					Log.i("PAUSESTART", String.valueOf(stime));
					Log.i("PAUSEEND", String.valueOf(etime));
					Log.i("PAUSEDURATION", String.valueOf(etime-stime));
					
					float px = myGraph.getPauseTimes().get(i).getX();
					float py = myGraph.getPauseTimes().get(i).getY();
					
					float pp = myGraph.getPauseTimes().get(i).getPressure();
					float ps = myGraph.getPauseTimes().get(i).getSize();
					
					pw.print(String.valueOf(stime));
					pw.print(",");
					pw.print(String.valueOf(etime));
					pw.print(",");
					pw.print(String.valueOf(px));
					pw.print("|");
					pw.print(String.valueOf(py));
					pw.print(",");
					pw.print(String.valueOf(pp));
					pw.print(",");
					pw.print(String.valueOf(ps));
					pw.print(",");

				}
				
				pw.print(myGraph.getPressureSum() / myGraph.getNumTouchPoints());
				pw.print(",");
				pw.print(myGraph.getSizeSum() / myGraph.getNumTouchPoints());
				pw.print(",");
				
				float pavg = myGraph.getPressureSum() / myGraph.getNumTouchPoints();
				float savg = myGraph.getSizeSum() / myGraph.getNumTouchPoints();
				
				float asum = 0;
				for(int i = 0; i < myGraph.getPressureSamples().size(); i++)
				{
					float p = myGraph.getPressureSamples().get(i);
					asum += Math.pow((p - pavg), 2);
				}
				
				pw.print(asum / myGraph.getNumTouchPoints());
				pw.print(",");
				
				asum = 0;
				for(int i = 0; i < myGraph.getSizeSamples().size(); i++)
				{
					float s = myGraph.getSizeSamples().get(i);
					asum += Math.pow((s - savg), 2);
				}
				
				pw.print(asum / myGraph.getNumTouchPoints());
				pw.print(",");
				
				for(int i = 0; i < myGraph.getNodes().size(); i++)
				{
					Node n = myGraph.getNodes().get(i);
					for(int j = 0; j < n.getTimeInfo().size(); j++)
					{
						NodeTime ti = n.getTimeInfo().get(j);
						double distance = Math.sqrt(Math.pow((ti.getExitX() - ti.getEnterX()), 2) + Math.pow((ti.getExitY() - ti.getEnterY()), 2));
						
						float time = (float) ((ti.getExitTimestamp() - ti.getEnterTimestamp()) / 1000000000.0);
						
						pw.print(distance/time);
						pw.print(",");
						pw.print(distance);
						pw.print(",");
					}
				}
				
				for(int i = 0; i < myGraph.getPressureDataInside().size(); i++)
				{
					float sum = 0;
					AvgTypeData p = myGraph.getPressureDataInside().get(i);
					pw.print(p.getAverage());
					pw.print(",");
					
					float avg = p.getAverage();
					
					for(int j = 0; j < p.getSamples().size(); j++)
					{
						float s = p.getSamples().get(j);
						sum += Math.pow((s - avg), 2);
					}
					
					pw.print(sum / p.getNumSamples());
					pw.print(",");
				}
				
				for(int i = 0; i < myGraph.getSizeDataInside().size(); i++)
				{
					float sum = 0;
					AvgTypeData s = myGraph.getSizeDataInside().get(i);
					pw.print(s.getAverage());
					pw.print(",");
					
					float avg = s.getAverage();
					
					for(int j = 0; j < s.getSamples().size(); j++)
					{
						float p = s.getSamples().get(j);
						sum += Math.pow((p - avg), 2);
					}
					
					pw.print(sum / s.getNumSamples());
					pw.print(",");
					
				}
				
				for(int i =0; i < myGraph.getPressureDataBetween().size(); i++)
				{
					float sum = 0;
					AvgTypeData p = myGraph.getPressureDataBetween().get(i);
					pw.print(p.getAverage());
					pw.print(",");
					
					float avg = p.getAverage();
					
					for(int j = 0; j < p.getSamples().size(); j++)
					{
						float s = p.getSamples().get(j);
						sum += Math.pow((s - avg), 2);
					}
					
					pw.print(sum / p.getNumSamples());
					pw.print(",");
				}
				
				for(int i = 0; i < myGraph.getSizeDataBetween().size(); i++)
				{
					float sum = 0;
					AvgTypeData s = myGraph.getSizeDataBetween().get(i);
					pw.print(s.getAverage());
					pw.print(",");
					
					float avg = s.getAverage();
					
					for(int j = 0; j < s.getSamples().size(); j++)
					{
						float p = s.getSamples().get(j);
						sum += Math.pow((p - avg), 2);
					}
					
					pw.print(sum / s.getNumSamples());
					pw.print(",");
				}
				
				for(int i = 0; i < myGraph.getRateDataBetween().size(); i++)
				{
					RateInfo r = myGraph.getRateDataBetween().get(i);
					double distance = Math.sqrt(Math.pow((r.getEndX()- r.getStartX()), 2) + Math.pow((r.getEndY() - r.getStartY()), 2));
					
					pw.print(distance/r.getTime());
					pw.print(",");
					pw.print(distance);
					pw.print(",");
				}
				
				pw.close();
				try {
					f.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			//TODO: handle not enough space later
	
		}
		
	}

}
