package com.example.trailsfinal;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.String;

import android.content.Context;
import android.graphics.Canvas;
import android.util.DisplayMetrics;
import android.util.Log;

import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;

public class Graph{
	
	//list of nodes
	private List<Node> nodes = new ArrayList();
	//list of edges
	private List<Edge> edges;
	//contains nodes and edges, edge initially empty since user will draw those
	//nodes should probably contain a path too for what is drawn inside them
	private List<String> currentStr;
	
	private List<PauseTime> liftTimes= new ArrayList<PauseTime>();

	private String trailsAStr = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20";
	private String trailsBStr = "1,A,2,B,3,C,4,D,5,E,6,F,7,G,8,H,9,I,10";
	private String trailsSStr = "1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,10,10";
	private String trailsS2Str = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20";
	private String pAStr = "1,2,3,4,5,6,7,8";
	private String pBStr = "1,A,2,B,3,C,4,D";
	private String pSStr = "1,1,2,2,3,3,4,4";
	private String pS2Str = "1,2,3,4,5,6,7,8";
	
	
	private float rad = 35;
	private String posA;
	private String posB;
	private String posS;
	private String posPA;
	private String posPB;
	private String posPS;
	private String posS2;
	private String posPS2;
	
	
	//put the basic time stuff first
	
	private int numCorrect = 0;
	
	private int numErrors = 0;
	//something for total time
	private long startTime = 0;
	
	private double totalTime = 0;
	
	private float numTouchPoints = 0;
	
	//total avg pressure
	private float pressureSum = 0;
	
	//total avg size
	private float sizeSum = 0;
	
	private List<Float> pressureSamples = new ArrayList<Float>();
	private List<Float> sizeSamples = new ArrayList<Float>();
	
	
	//air time vs down time
	private long totalAirtime = 0;
	private long totalDownTime = 0;
	
	private int currentNodeIndex = 0;
	
	private int canvasWidth = 0;
	private int canvasHeight = 0;
	private DisplayMetrics canvasdm;
	
	private String myType;
	
	private Context myCont;
	
	private List<PauseTime> pauseTimes = new ArrayList<PauseTime>();
	
	
	
	private List<RateInfo> rateDataInside = new ArrayList<RateInfo>();
	private List<AvgTypeData> pressureDataInside = new ArrayList<AvgTypeData>();
	private List<AvgTypeData> sizeDataInside = new ArrayList<AvgTypeData>();
	
	
	private List<RateInfo> rateDataBetween = new ArrayList<RateInfo>();
	private List<AvgTypeData> pressureDataBetween = new ArrayList<AvgTypeData>();
	private List<AvgTypeData> sizeDataBetween = new ArrayList<AvgTypeData>();
	
	
	
	//loads default trails A or B
	public Graph(String type, int CW, int CH, DisplayMetrics dm, Context cont)
	{
		canvasWidth = CW;
		canvasHeight = CH;
		canvasdm = dm;
		myType = type;
		myCont = cont;
		
		if(type.equals("A"))
		{
			posA = generatePos("A");
			loadTrails(trailsAStr, posA);
		}
		if(type.equals("B"))
		{
			posB = generatePos("B");
			loadTrails(trailsBStr, posB);
		}
		if(type.equals("S"))
		{
			posS = generatePos("S");
			loadTrails(trailsSStr, posS);
		}
		if(type.equals("S2"))
		{
			posS2 = generatePos("S2");
			loadTrails(trailsS2Str, posS2);
		}
		if(type.equals("PA"))
		{
			posPA = generatePos("PA");
			loadTrails(pAStr, posPA);
		}
		if(type.equals("PB"))
		{
			posPB = generatePos("PB");
			loadTrails(pBStr, posPB);
		}
		if(type.equals("PS"))
		{
			posPS = generatePos("PS");
			loadTrails(pSStr, posPS);
		}
		if(type.equals("PS2"))
		{
			posPS2 = generatePos("PS2");
			loadTrails(pS2Str, posPS2);
		}
	}
	
	public List<Float> getSizeSamples()
	{
		return sizeSamples;
	}
	
	public List<Float> getPressureSamples()
	{
		return pressureSamples;
	}
	
	public void addSizeSample(float s)
	{
		sizeSamples.add(s);
	}
	
	public void addPressureSample(float p)
	{
		pressureSamples.add(p);
	}
	
	public List<RateInfo> getRateDataInside()
	{
		return rateDataInside;
	}
	
	public List<RateInfo> getRateDataBetween()
	{
		return rateDataBetween;
	}
	
	public List<AvgTypeData> getPressureDataInside()
	{
		return pressureDataInside;
	}
	
	public List<AvgTypeData> getPressureDataBetween()
	{
		return pressureDataBetween;
	}
	
	public List<AvgTypeData> getSizeDataInside()
	{
		return sizeDataInside;
	}
	
	public List<AvgTypeData> getSizeDataBetween()
	{
		return sizeDataBetween;
	}

	
	public float getSizeSum()
	{
		return sizeSum;
	}
	
	public void setSizeSum(float ss)
	{
		sizeSum = ss;
	}
	
	public float getPressureSum()
	{
		return pressureSum;
	}
	
	public void setPressureSum(float sum)
	{
		pressureSum = sum;
	}
	
	public float getNumTouchPoints()
	{
		return numTouchPoints;
	}
	
	public void setNumTouchPoints(float num)
	{
		numTouchPoints = num;
	}
	
	public List<PauseTime> getLiftTimes()
	{
		return liftTimes;
	}
	
	public List<PauseTime> getPauseTimes()
	{
		return pauseTimes;
	}
	
	public void addLiftTime(long start, long end, float X, float Y)
	{
		PauseTime time = new PauseTime(start, end, X, Y);
		liftTimes.add(time);
	}
	
	public void addPressureDataInside(String startNode, String type)
	{
		AvgTypeData p = new AvgTypeData(startNode, type);
		pressureDataInside.add(p);
	}
	
	public void addPressureDataBetween(String startNode, String type)
	{
		AvgTypeData p = new AvgTypeData(startNode, type);
		pressureDataBetween.add(p);
	}
	
	public void addSizeDataInside(String startNode, String type)
	{
		AvgTypeData s = new AvgTypeData(startNode, type);
		sizeDataInside.add(s);
	}
	
	public void addSizeDataBetween(String startNode, String type)
	{
		AvgTypeData s = new AvgTypeData(startNode, type);
		sizeDataBetween.add(s);
	}
	
	public void addRateDataInside(String startNode, float startx, float starty, float starttime, String type)
	{
		RateInfo r = new RateInfo(startNode, startx, starty, starttime, type);
		rateDataInside.add(r);
	}
	
	public void addRateDataBetween(String startNode, float startx, float starty, float starttime, String type)
	{
		RateInfo r = new RateInfo(startNode, startx, starty, starttime, type);
		rateDataBetween.add(r);
	}
	
	public void addPauseTime(long start, long end, float X, float Y)
	{
		PauseTime time = new PauseTime(start, end, X, Y);
		pauseTimes.add(time);
	}
	
	public String getType()
	{
		return myType;
	}
	
	public int getCurrentIndex()
	{
		return currentNodeIndex;
	}
	
	public void setCurrentIndex(Node selectedNode)
	{
		currentNodeIndex = currentStr.indexOf(selectedNode.getCharacter());
	}
	
	public List<Node> getNodes()
	{
		return nodes;
	}
	
	public int getNumCorrect()
	{
		return numCorrect;
	}
	
	public void setNumCorrect(Node selectedNode, int num)
	{
		Node correctNode = nodes.get(num);
		if(selectedNode.getIndex() == correctNode.getIndex())
		{
			numCorrect += 1;
		}
	}
	
	public double getTotalTime()
	{
		return totalTime;
	}
	
	public long getStartTime()
	{
		return startTime;
	}
	
	public int getNumErrors()
	{
		return numErrors;
	}
	
	public void setNumErrors(int num)
	{
		numErrors = num;
	}
	
	public void setStartTime(long start)
	{
		startTime = start;
	}
	
	
	
	public String generatePos(String type)
	{
		StringBuilder sb = new StringBuilder();
		String file = "";
		int startx = (canvasWidth/2);
		int starty = canvasHeight/2;
		double xfactor = 1.0;
		double yfactor = 1.0;
		
		if(type.equals("PB"))
		{
			file = "PBPos.txt";
			
			//startx = (int)((canvasWidth / 2) - (canvasWidth / 12));
		}
		if(type.equals("PA"))
		{
			file = "PAPos.txt";
			
			//startx = (int)((canvasWidth / 2) - (canvasWidth / 10));
			//starty = (canvasHeight / 2);
			
		
		}
		if(type.equals("PS"))
		{
			file = "PSPos.txt";
			
			//startx = (int)((canvasWidth / 2) - (canvasWidth / 10));
			//starty = (canvasHeight / 2);
			
		
		}
		if(type.equals("PS2"))
		{
			file = "PS2Pos.txt";
			
			//startx = (int)((canvasWidth / 2) - (canvasWidth / 10));
			//starty = (canvasHeight / 2);
			
		
		}
		if(type.equals("A"))
		{
			file = "PosA.txt";
			
			//startx = (int)((canvasWidth / 2) + (canvasWidth / 3.9));
			starty = (canvasHeight / 2) - (canvasHeight / 30);
			xfactor = 0.65;
			yfactor = 0.75;
		}
		if(type.equals("B"))
		{
			file = "PosB.txt";
			//startx = (canvasWidth / 2) - (canvasWidth / 17);
			//startx = (canvasWidth / 2) + (canvasWidth / 13);
			//starty = (canvasHeight / 2) - (canvasWidth / 17);
			//starty = (canvasHeight / 2);
			xfactor = 0.64;
			yfactor = 0.75;
		}
		if(type.equals("S"))
		{
			file = "PosS.txt";
			//startx = (canvasWidth / 2) - (canvasWidth / 17);
			//startx = (canvasWidth / 2) + (canvasWidth / 13);
			//starty = (canvasHeight / 2) - (canvasWidth / 17);
			//starty = (canvasHeight / 2);
			xfactor = 0.64;
			yfactor = 0.75;
		}
		if(type.equals("S2"))
		{
			file = "PosS2.txt";
			//startx = (canvasWidth / 2) - (canvasWidth / 17);
			//startx = (canvasWidth / 2) + (canvasWidth / 13);
			//starty = (canvasHeight / 2) - (canvasWidth / 17);
			//starty = (canvasHeight / 2);
			xfactor = 0.64;
			yfactor = 0.75;
		}
		
		/*if(!file.equals("PosB.txt"))
		{
		sb.append(startx);
		sb.append(",");
		sb.append(starty);
		sb.append(":");
		}*/
		
		InputStream in = null;
		String line = "";
		try {
			in = myCont.getAssets().open(file);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		BufferedReader reader = new BufferedReader(new InputStreamReader(in));
		try {
			line = reader.readLine();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		do
		{
			try {
				
				//Log.i("Trails", line);
				List<String> values = Arrays.asList(line.split(","));
				double x = Double.parseDouble(values.get(0));
				double y = Double.parseDouble(values.get(1));
				
				/*Log.i("Trails", String.valueOf(x));
				Log.i("Trails", String.valueOf(y));
				Log.i("Trails", "Here are dpi's");
				Log.i("Trails", String.valueOf(canvasdm.xdpi));
				Log.i("Trails", String.valueOf(canvasdm.ydpi));*/
				
				
				//int nextx;
				//int nexty;
				//int nextx = (int) (startx + Math.cos(Math.toRadians(angle)) * ((distance - xshift) * canvasdm.xdpi));
				/*if(!file.equals("PosB.txt"))
				{
					nextx = (int) (startx + Math.cos(Math.toRadians(angle)) * ((distance*xfactor) * canvasdm.xdpi));
					nexty = (int) (starty - Math.sin(Math.toRadians(angle)) * ((distance*yfactor) * canvasdm.ydpi));
				}*/
				//nextx = (int) x;
			    //nexty = (int) y;
				//int nextx = (int) (startx + Math.cos(Math.toRadians(angle)) * ((distance*xfactor) * 115));
				//int nexty = (int) (starty - Math.sin(Math.toRadians(angle)) * ((distance*yfactor) * 115));
				//int nexty = (int) (starty - Math.sin(Math.toRadians(angle)) * ((distance - yshift) * canvasdm.ydpi));
				
				//startx = nextx;
				//starty = nexty;
				
				
				sb.append((int)x);
				sb.append(",");
				sb.append((int)y);
				sb.append(":");
				line = reader.readLine();
				
				
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		while(line != null);
		
		Log.i("Trails", type);
		Log.i("Trails", "PositionsDone");
		Log.i("Trails", sb.toString());
		return sb.toString();
	}
	
	//checks to see if the newly selected node is in the right order
	//looks at the node and the number of previous correctly 
	//selected nodes
	public boolean inOrder(Node selectedNode, int num)
	{
		Node correctNode = nodes.get(num);
		Log.i("Trails", "CorrectNode");
		Log.i("Trails", correctNode.getCharacter());
		Log.i("Trails", "SelectedNode");
		Log.i("Trails", selectedNode.getCharacter());
		
		if(selectedNode.getIndex() > correctNode.getIndex())
		{
			//System.out.println("Hello");
			return false;
		}
		return true;
	}

	private void loadTrails(String trailsStr, String posStr)
	{
		String c;
		Node n;
		
		String[]positions;
		String[]coords;
		String[]symbols;
		
		currentStr = Arrays.asList(trailsStr.split(","));
		if(posStr == "")
		{
			return;
		}
		
		positions = posStr.split(":");
		symbols = trailsStr.split(",");
		//make sure they are the same length
		//loop through trails A string, create nodes with those characters
		//real positions???
		for(int i = 0; i < symbols.length; i++)
		{
			c = symbols[i];
			coords = positions[i].split(",");
			if(!this.myType.equals("S") && !this.myType.equals("PS") && !this.myType.equals("PS2") && !this.myType.equals("S2"))
			{
				n = new Node(rad, Integer.parseInt(coords[0]), Integer.parseInt(coords[1]), c, i, "C");
				nodes.add(n);
			}
			else
			{
				n = new Node(rad, Integer.parseInt(coords[0]), Integer.parseInt(coords[1]), c, i, "S");
				nodes.add(n);
			}
			
		}
		
	}
	
	public long trailsComplete(int numCorrect)
	{
		if(numCorrect == currentStr.size())
		{
			Log.i("Trails", "TrailsComplete");
			Log.i("Trails", String.valueOf(numCorrect));
			return System.nanoTime();
		}
		return 0;
	}
	
	//Call at the end of the game
	//Call between nodes and store result in an array
	public String getDuration(long endTime)
   {
		        Log.i("Trails", String.valueOf(startTime));
		    	String durationStr = "";
		    	if(startTime != 0 && endTime != 0)
		    	{
		    		//build the str
		    		long duration = endTime - startTime;
		    		totalTime = (endTime - startTime)/1000000000.0;
		    		Log.i("Trails", "Done");
		    		Log.i("Trails", String.valueOf(duration));
		    	}
		    	
		    	return durationStr;
		    	
	}
	
	
	//take a string and generate a random graph of nodes with those characters
	//or maybe just generate a random graph of its own
	void generateRandom(int numNodes)
	{
	}

	//also generate graph of colored nodes
	void generateColored()
	{
	}
	
	

}
